<?php

namespace Drupal\dynamic_routes;

use Symfony\Component\Routing\Route;
use Drupal\Core\Session\AccountProxy;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\ParamConverter\ParamConverterInterface;

/**
 * Does the things.
 */
class ParamConverter implements ParamConverterInterface {

  /**
   * The entity type manager service.
   *
   * @var Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The current user.
   *
   * @var Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * Constructs a new ParamConverter.
   *
   * @param Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param Drupal\Core\Session\AccountProxy $current_user
   *   The language manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountProxy $current_user) {
    $this->entityTypeManager = $entity_type_manager;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public function convert($value, $definition, $name, array $defaults) {
    if ($definition["type"] === 'entity:user') {
      $id = $value === '0' && !$this->currentUser->isAnonymous() ? $this->currentUser->id() : $value;
      if ($id === 0 || $id === '0' || !empty($id)) {
        $entity = $this->entityTypeManager->getStorage('user')->load($id);
        if (!empty($entity)) {
          return $entity;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function applies($definition, $name, Route $route) {
    return isset($definition["type"]) && $definition["type"] === 'entity:user';
  }

}
